/*-
 * Copyright (c) 2002    Granch Ltd. 	All rights reserved.
 *
 * All or some portions of this file are derived from material licensed
 * to the Grahcn Ltd. and are reproduced herein with the permission of
 * Granch Ltd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE GRANCH LTD. AND THEIR CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author : Rashid N. Achilov 		E-Mail shelton@sentry.granch.ru
 *
 *	$Id: localtypes.h,v 0.94.8.2 2005/03/03 14:58:53 shelton Exp $
 */
/*-------------------------------------------------------------------------

            Spam Control and Canceling mail filter, based on libmilter API

	    UNIX FreeBSD 4.6 version

            Global types definition.

-------------------------------------------------------------------------*/

struct local_flags			// Local flags structure
 {
   unsigned	 _openlog		:1;	// System log is opened
   unsigned	 _pidfile		:1;	// PID file is presented
   unsigned	 _alloc_config  	:1;	// Memory to config file allocated
   unsigned	 _imitate		:1;	// Imitate only

   unsigned	 _nodelete		:1;	// No delete any mail anyway
   unsigned	 _spam_nodelete		:1;	// No delete spam mail anyway
   unsigned	 _res1			:1;	// Reserved bit
   unsigned	 _res2			:1;	// Reserved bit
 };

struct conn_flags			// Connection flag structure
 {
   unsigned	_isspam		:1;	// This mail is spam-suspected
   unsigned 	_firstheader	:1;	// First header line already was
   unsigned	_tempsave	:1;	// Open temp/save header file
   unsigned	_firstrcpt	:1;	// First recipient already was given
//
   unsigned	_isenterviol	:1;	// This mail violated enterprise rules
   unsigned	_htmlfrom	:1;	// Allow to receive HTML messages
   unsigned	_localout	:1;	// Local user mail out
   unsigned	_multipart	:1;	// This is a multipart message
//
   unsigned	_charset	:1;	// charset= was found in Content-Type
   unsigned	_boundary	:1;	// boundary= was found in Content-Type
   unsigned	_bstop		:1;	// boundary stopper was found
   unsigned	_noctparam	:1;	// No parameters followed after media type
//
   unsigned	_tempbody	:1;	// Open temp/save body file
   unsigned	_res2		:1;	// Reserved bit
   unsigned	_res3		:1;	// Reserved bit
   unsigned	_res4		:1;	// Reserved bit
 };

struct bnd_flags			// Boundary flag structure
 {
   unsigned	_noctparam	:1;	// Content-Type has only type/subtype
   unsigned	_image		:1;	// Media type is image
   unsigned	_audio		:1;	// Media type is audio
   unsigned	_video		:1;	// Media type is video
//
   unsigned	_application	:1;	// Media type is application
   unsigned	_multipart	:1;	// This boundary contains multipart message
   unsigned	_boundary	:1;	// boundary= was found
   unsigned	_text		:1;	// Media type is text
//
   unsigned	_incomplete	:1;	// MIME header of boundary was not finished
   unsigned	_iscontype	:1;	// Content-Type was not finished yet
   unsigned	_media		:1;	// Media type was parsed
   unsigned	_res12		:1;	// Reserved bit
//
   unsigned	_res13		:1;	// Reserved bit
   unsigned	_res14		:1;	// Reserved bit
   unsigned	_res15		:1;	// Reserved bit
   unsigned	_res16		:1;	// Reserved bit
 };

struct send_flags
 {
   unsigned	_htmlin		:1;	// Allow to receive HTML messages
   unsigned	_htmlout	:1;	// Allow to send HTML messages
   unsigned	_pgp		:1;	// Allow to send/receive PGP encrypted mail
   unsigned     _exattach	:1;	// Allow to receive executable attachments

   unsigned	_attach		:1;	// Allow to receive any other attachments
   unsigned	_ischarset	:1;	// This sender has defined charset table
   unsigned			:2;	// Reserved bits
 };

struct recv_flags
 {
   unsigned	_htmlin		:1;	// Allow to receive HTML messages
   unsigned	_htmlout	:1;	// Allow to sendL messages
   unsigned	_pgp		:1;	// Allow to send/receive PGP encrypted mail
   unsigned     _exattach	:1;	// Allow to receive executable attachments

   unsigned	_attach		:1;	// Allow to receive any other attachments
   unsigned	_ischarset	:1;	// This recipient has defined charset table
   unsigned			:2;	// Reserved bits
 };

struct host_flags
 {
   unsigned	_html		:1;	// Allow to receive HTML messages
   unsigned			:7;	// reserved bits
 };

struct user_flags
 {
   unsigned	_htmlin		:1;	// Allow to receive HTML messages
   unsigned	_htmlout	:1;	// Allow to send HTML messages
   unsigned	_pgp		:1;	// Allow to send/receive PGP encrypted mail
   unsigned     _exattach	:1;	// Allow to receive executable attachments
//
   unsigned	_attach		:1;	// Allow to receive any other attachments
   unsigned	_ischarset	:1;	// This recipient has defined charset table
   unsigned	_res1		:1;	// Reserved bits
   unsigned	_res2		:1;	// Reserved bits
 };

struct mlfiPriv				// Mail filter private data storage
 {
   char    	     *mlfi_fname;	// Temp/save header filename
   char 	     *mlfi_bname;	// Temp/save body filename
   char    	     *mlfi_connectfrom; // Connect from: container
   struct sccuser    *mlfi_mailfrom;	// MAIL FROM: container
   char    	     *mlfi_helofrom;	// HELO from: container
   struct sccuser    *mlfi_rcptto;	// RCPT TO:'s container
   FILE    	     *mlfi_fp;	    	// File handle to keep header
   FILE		     *mlfi_fpbody;	// File handle to keep body
   struct in_addr    inaddr;		// Host IP in hex form (4 bytes)
   struct tm 	     *ptm;		// Time containver
   struct conn_flags _cflags;		// Connection flags
   struct bnd_flags  _bflags;		// Boundary flags
   short int         recipients;	// Number of recipients in mail
   short int	     headerlines;	// Header lines counter
   short int	     body_parts;	// Mail body can be split in a chunks
   int		     body_length;	// Message body (accomplished) length
   short int	     charsetlength;	// Length of value in charset= parameter
   short int	     boundarylength;	// Length of value in boundary= parameter
   char		     *work;		// Workspace to codepages, etc.
   char		     *charset;		// Pointer to charset storage area
   char		     *boundary;		// Pointer to boundary copy
 };

struct config_flags			// Config file flags structure
 {
   unsigned _isdaemon		:1;  	// Go to daemon mode
   unsigned _mode_isolate	:1;  	// Default mode - ISOLATE
   unsigned _mode_discard	:1;  	// Default mode - DISCARD
   unsigned _emode_isolate 	:1;	// Enterprise mode - ISOLATE

   unsigned _emode_reject  	:1;	// Enterprise mode - REJECT
   unsigned _nohtmlin		:1;	// Isolating incoming HTML mail
   unsigned _nohtmlout		:1;	// Isolating outgoing HTML mail
   unsigned _enterprise		:1;	// Enterprise violations control

   unsigned _nocharset		:1;	// No charset issued spam action
   unsigned			:7;	// Reserved bits
 };

struct local_config			// Config file description structure
 {
   char *progname;			// Mandatory field - program name
   char *progversion;			// Mandatory field - program version
   char *sendmail_pipe;			// Pipe to use with sendmail
   char *scchosts;			// SCC Hosts file
   char *sccusers;			// SCC users file
   char *pidfile;			// PID number keeper
   char *isolation;			// Directory to keep mail (isolator)
   char *codepages;			// Allowed codepages
   char *domain;			// Local domain parameter
   short int sendmail_timeout;		// Timeout to connect to sendmail
   short int debug_level;		// Level of debugging messages
   struct config_flags _idflags;	// Config file flags
 };

struct scchost
 {
   struct in_addr scc_addr;		// Host IP in hex form (4 bytes)
   struct host_flags _hflags;		// Host flags
 };

struct sccuser
 {
   char sccuser[81];		  	// Username (max 80 chars + zero pad)
   struct user_flags _uflags;		// User flags
   char sccpages[61];			// THis user's allowed codepages
 };
